=== modified file 'plugins.d/mandosclient.c'
--- plugins.d/mandosclient.c	2008-08-02 15:47:42 +0000
+++ plugins.d/mandosclient.c	2008-08-03 14:57:46 +0000
@@ -32,6 +32,8 @@
 #define _LARGEFILE_SOURCE
 #define _FILE_OFFSET_BITS 64
 
+#define _GNU_SOURCE		/* TEMP_FAILURE_RETRY() */
+
 #include <stdio.h>
 #include <assert.h>
 #include <stdlib.h>
@@ -77,12 +79,27 @@
 
 bool debug = false;
 
+const char mandos_protocol_version[] = "1";
+
 typedef struct {
-  gnutls_session_t session;
+  AvahiSimplePoll *simple_poll;
+  AvahiServer *server;
   gnutls_certificate_credentials_t cred;
-  gnutls_dh_params_t dh_params;
-} encrypted_session;
+  unsigned int dh_bits;
+  const char *priority;
+} mandos_context;
 
+size_t adjustbuffer(char *buffer, size_t buffer_length,
+		  size_t buffer_capacity){
+  if (buffer_length + BUFFER_SIZE > buffer_capacity){
+    buffer = realloc(buffer, buffer_capacity + BUFFER_SIZE);
+    if (buffer == NULL){
+      return 0;
+    }
+    buffer_capacity += BUFFER_SIZE;
+  }
+  return buffer_capacity;
+}
 
 static ssize_t pgp_packet_decrypt (char *packet, size_t packet_size,
 				   char **new_packet,
@@ -91,7 +108,7 @@
   gpgme_ctx_t ctx;
   gpgme_error_t rc;
   ssize_t ret;
-  ssize_t new_packet_capacity = 0;
+  size_t new_packet_capacity = 0;
   ssize_t new_packet_length = 0;
   gpgme_engine_info_t engine_info;
 
@@ -204,12 +221,10 @@
   
   *new_packet = 0;
   while(true){
-    if (new_packet_length + BUFFER_SIZE > new_packet_capacity){
-      *new_packet = realloc(*new_packet,
-			    (unsigned int)new_packet_capacity
-			    + BUFFER_SIZE);
-      if (*new_packet == NULL){
-	perror("realloc");
+    new_packet_capacity = adjustbuffer(*new_packet, new_packet_length,
+				       new_packet_capacity);
+    if (new_packet_capacity == 0){
+	perror("adjustbuffer");
 	return -1;
       }
       new_packet_capacity += BUFFER_SIZE;
@@ -253,7 +268,7 @@
   fprintf(stderr, "%s", string);
 }
 
-static int initgnutls(encrypted_session *es){
+static int initgnutls(mandos_context *mc){
   const char *err;
   int ret;
   
@@ -322,7 +337,7 @@
 	    safer_gnutls_strerror(ret));
   }
   
-  if ((ret = gnutls_priority_set_direct (es->session, "NORMAL", &err))
+  if ((ret = gnutls_priority_set_direct (es->session, mc->priority, &err))
       != GNUTLS_E_SUCCESS) {
     fprintf(stderr, "Syntax error at: %s\n", err);
     fprintf(stderr, "GnuTLS error: %s\n",
@@ -351,7 +366,8 @@
 		      __attribute__((unused)) const char *txt){}
 
 static int start_mandos_communication(const char *ip, uint16_t port,
-				      AvahiIfIndex if_index){
+				      AvahiIfIndex if_index,
+				      mandos_context *mc){
   int ret, tcp_sd;
   struct sockaddr_in6 to;
   encrypted_session es;
@@ -360,7 +376,7 @@
   size_t buffer_length = 0;
   size_t buffer_capacity = 0;
   ssize_t decrypted_buffer_size;
-  size_t written = 0;
+  size_t written;
   int retval = 0;
   char interface[IF_NAMESIZE];
   
@@ -418,7 +434,31 @@
     perror("connect");
     return -1;
   }
-  
+
+  char *out = mandos_protocol_version;
+  written = 0;
+  while (true){
+    size_t out_size = strlen(out);
+    ret = TEMP_FAILURE_RETRY(write(tcp_sd, out + written,
+				   out_size - written));
+    if (ret == -1){
+      perror("write");
+      retval = -1;
+      goto end;
+    }
+    written += ret;
+    if(written < out_size){
+      continue;
+    } else {
+      if (out == mandos_protocol_version){
+	written = 0;
+	out = "\r\n";
+      } else {
+	break;
+      }
+    }
+  }
+ 
   ret = initgnutls (&es);
   if (ret != 0){
     retval = -1;
@@ -451,13 +491,11 @@
   }
 
   while(true){
-    if (buffer_length + BUFFER_SIZE > buffer_capacity){
-      buffer = realloc(buffer, buffer_capacity + BUFFER_SIZE);
-      if (buffer == NULL){
-	perror("realloc");
-	goto exit;
-      }
-      buffer_capacity += BUFFER_SIZE;
+    buffer_capacity = adjustbuffer(buffer, buffer_length, buffer_capacity);
+    if (buffer_capacity == 0){
+      perror("adjustbuffer");
+      retval = -1;
+      goto exit;
     }
     
     ret = gnutls_record_recv
@@ -497,6 +535,7 @@
 					       &decrypted_buffer,
 					       certdir);
     if (decrypted_buffer_size >= 0){
+      written = 0;
       while(written < (size_t) decrypted_buffer_size){
 	ret = (int)fwrite (decrypted_buffer + written, 1,
 			   (size_t)decrypted_buffer_size - written,
@@ -533,24 +572,20 @@
   return retval;
 }
 
-static AvahiSimplePoll *simple_poll = NULL;
-static AvahiServer *server = NULL;
-
-static void resolve_callback(
-    AvahiSServiceResolver *r,
-    AvahiIfIndex interface,
-    AVAHI_GCC_UNUSED AvahiProtocol protocol,
-    AvahiResolverEvent event,
-    const char *name,
-    const char *type,
-    const char *domain,
-    const char *host_name,
-    const AvahiAddress *address,
-    uint16_t port,
-    AVAHI_GCC_UNUSED AvahiStringList *txt,
-    AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
-    AVAHI_GCC_UNUSED void* userdata) {
-    
+static void resolve_callback( AvahiSServiceResolver *r,
+			      AvahiIfIndex interface,
+			      AVAHI_GCC_UNUSED AvahiProtocol protocol,
+			      AvahiResolverEvent event,
+			      const char *name,
+			      const char *type,
+			      const char *domain,
+			      const char *host_name,
+			      const AvahiAddress *address,
+			      uint16_t port,
+			      AVAHI_GCC_UNUSED AvahiStringList *txt,
+			      AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
+			      AVAHI_GCC_UNUSED void* userdata) {
+  mandos_context *mc = userdata;
   assert(r);			/* Spurious warning */
   
   /* Called whenever a service has been resolved successfully or
@@ -561,7 +596,7 @@
   case AVAHI_RESOLVER_FAILURE:
     fprintf(stderr, "(Resolver) Failed to resolve service '%s' of"
 	    " type '%s' in domain '%s': %s\n", name, type, domain,
-	    avahi_strerror(avahi_server_errno(server)));
+	    avahi_strerror(avahi_server_errno(mc->server)));
     break;
     
   case AVAHI_RESOLVER_FOUND:
@@ -572,7 +607,7 @@
 	fprintf(stderr, "Mandos server \"%s\" found on %s (%s) on"
 		" port %d\n", name, host_name, ip, port);
       }
-      int ret = start_mandos_communication(ip, port, interface);
+      int ret = start_mandos_communication(ip, port, interface, mc);
       if (ret == 0){
 	exit(EXIT_SUCCESS);
       }
@@ -581,53 +616,51 @@
   avahi_s_service_resolver_free(r);
 }
 
-static void browse_callback(
-    AvahiSServiceBrowser *b,
-    AvahiIfIndex interface,
-    AvahiProtocol protocol,
-    AvahiBrowserEvent event,
-    const char *name,
-    const char *type,
-    const char *domain,
-    AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
-    void* userdata) {
-    
-    AvahiServer *s = userdata;
-    assert(b);			/* Spurious warning */
-    
-    /* Called whenever a new services becomes available on the LAN or
-       is removed from the LAN */
-    
-    switch (event) {
-    default:
-    case AVAHI_BROWSER_FAILURE:
-      
-      fprintf(stderr, "(Browser) %s\n",
-	      avahi_strerror(avahi_server_errno(server)));
-      avahi_simple_poll_quit(simple_poll);
-      return;
-      
-    case AVAHI_BROWSER_NEW:
-      /* We ignore the returned resolver object. In the callback
-	 function we free it. If the server is terminated before
-	 the callback function is called the server will free
-	 the resolver for us. */
-      
-      if (!(avahi_s_service_resolver_new(s, interface, protocol, name,
-					 type, domain,
-					 AVAHI_PROTO_INET6, 0,
-					 resolve_callback, s)))
-	fprintf(stderr, "Failed to resolve service '%s': %s\n", name,
-		avahi_strerror(avahi_server_errno(s)));
-      break;
-      
-    case AVAHI_BROWSER_REMOVE:
-      break;
-      
-    case AVAHI_BROWSER_ALL_FOR_NOW:
-    case AVAHI_BROWSER_CACHE_EXHAUSTED:
-      break;
-    }
+static void browse_callback( AvahiSServiceBrowser *b,
+			     AvahiIfIndex interface,
+			     AvahiProtocol protocol,
+			     AvahiBrowserEvent event,
+			     const char *name,
+			     const char *type,
+			     const char *domain,
+			     AVAHI_GCC_UNUSED AvahiLookupResultFlags flags,
+			     void* userdata) {
+  mandos_context *mc = userdata;
+  assert(b);			/* Spurious warning */
+  
+  /* Called whenever a new services becomes available on the LAN or
+     is removed from the LAN */
+  
+  switch (event) {
+  default:
+  case AVAHI_BROWSER_FAILURE:
+      
+    fprintf(stderr, "(Browser) %s\n",
+	    avahi_strerror(avahi_server_errno(mc->server)));
+    avahi_simple_poll_quit(mc->simple_poll);
+    return;
+      
+  case AVAHI_BROWSER_NEW:
+    /* We ignore the returned resolver object. In the callback
+       function we free it. If the server is terminated before
+       the callback function is called the server will free
+       the resolver for us. */
+      
+    if (!(avahi_s_service_resolver_new(mc->server, interface, protocol, name,
+				       type, domain,
+				       AVAHI_PROTO_INET6, 0,
+				       resolve_callback, mc)))
+      fprintf(stderr, "Failed to resolve service '%s': %s\n", name,
+	      avahi_strerror(avahi_server_errno(s)));
+    break;
+      
+  case AVAHI_BROWSER_REMOVE:
+    break;
+      
+  case AVAHI_BROWSER_ALL_FOR_NOW:
+  case AVAHI_BROWSER_CACHE_EXHAUSTED:
+    break;
+  }
 }
 
 /* Combines file name and path and returns the malloced new
@@ -662,6 +695,8 @@
     int sd;
     char *connect_to = NULL;
     AvahiIfIndex if_index = AVAHI_IF_UNSPEC;
+    mandos_context mc = { .simple_poll = NULL, .server = NULL,
+			  .dh_bits = 2048, .priority = "SECURE256"};
     
     while (true){
       static struct option long_options[] = {
@@ -671,6 +706,8 @@
 	{"certdir", required_argument, 0, 'd'},
 	{"certkey", required_argument, 0, 'c'},
 	{"certfile", required_argument, 0, 'k'},
+	{"dh_bits", required_argument, 0, 'D'},
+	{"priority", required_argument, 0, 'p'},
 	{0, 0, 0, 0} };
       
       int option_index = 0;
@@ -699,6 +736,21 @@
       case 'k':
 	certkey = optarg;
 	break;
+      case 'D':
+	{
+	  long int tmp;
+	  errno = 0;
+	  tmp = strtol(optarg, NULL, 10);
+	  if (errno == ERANGE){
+	    perror("strtol");
+	    exit(EXIT_FAILURE);
+	  }
+	  mc.dh_bits = tmp;
+	}
+	break;
+      case 'p':
+	mc.priority = optarg;
+	break;
       default:
 	exit(EXIT_FAILURE);
       }
@@ -781,7 +833,7 @@
     srand((unsigned int) time(NULL));
 
     /* Allocate main loop object */
-    if (!(simple_poll = avahi_simple_poll_new())) {
+    if (!(mc.simple_poll = avahi_simple_poll_new())) {
         fprintf(stderr, "Failed to create simple poll object.\n");
 	returncode = EXIT_FAILURE;	
         goto exit;
@@ -795,14 +847,14 @@
     config.publish_domain = 0;
 
     /* Allocate a new server */
-    server = avahi_server_new(avahi_simple_poll_get(simple_poll),
+    mc.server = avahi_server_new(avahi_simple_poll_get(simple_poll),
 			      &config, NULL, NULL, &error);
 
     /* Free the configuration data */
     avahi_server_config_free(&config);
 
     /* Check if creating the server object succeeded */
-    if (!server) {
+    if (!mc.server) {
         fprintf(stderr, "Failed to create server: %s\n",
 		avahi_strerror(error));
 	returncode = EXIT_FAILURE;
@@ -810,13 +862,13 @@
     }
     
     /* Create the service browser */
-    sb = avahi_s_service_browser_new(server, if_index,
+    sb = avahi_s_service_browser_new(mc.server, if_index,
 				     AVAHI_PROTO_INET6,
 				     "_mandos._tcp", NULL, 0,
-				     browse_callback, server);
+				     browse_callback, &mc);
     if (!sb) {
         fprintf(stderr, "Failed to create service browser: %s\n",
-		avahi_strerror(avahi_server_errno(server)));
+		avahi_strerror(avahi_server_errno(mc.server)));
 	returncode = EXIT_FAILURE;
         goto exit;
     }
@@ -839,8 +891,8 @@
     if (sb)
         avahi_s_service_browser_free(sb);
     
-    if (server)
-        avahi_server_free(server);
+    if (mc.server)
+        avahi_server_free(mc.server);
 
     if (simple_poll)
         avahi_simple_poll_free(simple_poll);

=== modified file 'server.py'
--- server.py	2008-07-31 19:48:05 +0000
+++ server.py	2008-08-03 16:08:20 +0000
@@ -433,6 +433,15 @@
     def handle(self):
         logger.debug(u"TCP connection from: %s",
                      unicode(self.client_address))
+
+        line = self.socket.makefile().readline()
+        try:
+            if int(line.strip().split()[0]) > 1:
+                raise RuntimeError
+        except (ValueError, IndexError, RuntimeError), error:
+            logger.error(u"Unknown protocol version: %s", error)
+            return
+        
         session = gnutls.connection.ClientSession\
                   (self.request, gnutls.connection.X509Credentials())
         # Note: gnutls.connection.X509Credentials is really a generic

